/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package powermock.classloading.classes;

import java.lang.reflect.Method;

public class ReflectionMethodInvoker {
	private final Method method;
	private final Object instance;

	public ReflectionMethodInvoker(Method method, Object instance) {
		this.method = method;
		this.instance = instance;
	}

	public Object invoke() {
		try {
			return method.invoke(instance);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

}
