/*
 *  Copyright (c) 2007 Mockito contributors
 *  This program is made available under the terms of the MIT License.
 */

package org.powermock.api.mockito.repackaged.cglib.transform;

/**
 *
 * @author  baliuka
 */
public interface ClassFilter {
    
    boolean accept(String className);
    
}
