/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package demo.org.powermock.examples.simple;


/**
 * Hello world!
 *
 */
public class Greeter {
    public static void main( String[] args ) {
        new Greeter().run(5, getMessage());
    }

    private void run(int count, String message) {
        Logger logger = new Logger();
        for(int indx=0; indx<count; indx++){
			logger.log(message);
        }
    }

    private static String getMessage() {
		return SimpleConfig.getGreeting() + " " + SimpleConfig.getTarget();
	}
}
