package samples.powermockito.junit4.bugs.github352;

import org.junit.Test;

/**
 *
 */
public class MyTest extends MyAbstractTest {

    //some setup code here

    @Override
    public void test1() {
        //some test code here
    }

    @Override
    public void test2() {
        //some test code here
    }

    @Test
    public void test3() {
        // some test code here
    }

}
