/*
 *  Copyright (c) 2007 Mockito contributors
 *  This program is made available under the terms of the MIT License.
 */

/**
 * Mockito related CGLIB stuff. Repackaged from Mockito 2.0.43-beta with public visibility and cglib.
 */
package org.powermock.api.mockito.repackaged;
