/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package samples.junit4.legacy;

import org.junit.runner.RunWith;
import org.junit.runners.Suite;
import org.junit.runners.Suite.SuiteClasses;
import samples.junit4.legacy.noannotation.NoAnnotationUsageTest;
import samples.junit4.legacy.singleton.MockStaticTest;
import samples.junit4.legacy.suppressconstructor.SuppressConstructorHierarchyDemoTest;

@RunWith(Suite.class)
@SuiteClasses( { MockStaticTest.class, SuppressConstructorHierarchyDemoTest.class, NoAnnotationUsageTest.class })
public class AllJUnit4LegacyTests {
}
