/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package samples.simplemix;

/**
 * This class is used to demonstrate features such as static, final and partial
 * mocking as well as access internal state.
 */
public class SimpleMix {

    private SimpleMixCollaborator collaborator;

    public final int calculate() {
        return (SimpleMixUtilities.getRandomInteger() + getValue() - collaborator.getRandomInteger()) * new SimpleMixConstruction().getMyValue();
    }

    private int getValue() {
        return (int) (System.currentTimeMillis() / 1000);
    }
}
