/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package powermock.examples.jmock;

/**
 * Simple class that uses a static method and a final class.
 */
public class JMockExample {
    private final FinalClass finalClass;

    public JMockExample(FinalClass finalClass) {
        this.finalClass = finalClass;
    }

    public String generateMessage() {
        return "Message is: "+ finalClass.helloWorld() + ClassWithStaticMethod.returnString();
    }
}
