/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package samples.testng.agent;

import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.testng.PowerMockObjectFactory;
import org.testng.IObjectFactory;
import org.testng.annotations.ObjectFactory;
import org.testng.annotations.Test;
import samples.servletmocking.SampleServlet;

import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;

import static org.easymock.EasyMock.expect;
import static org.powermock.api.easymock.PowerMock.*;

@PrepareForTest(SampleServlet.class)
public class SampleServletTest {

    @Test
    public void doGet() throws Exception {
        SampleServlet servlet = new SampleServlet();

        HttpServletResponse response = createMock(HttpServletResponse.class);
        PrintWriter writer = createMock(PrintWriter.class);

        expect(response.getWriter()).andReturn(writer);
        writer.write("out");

        replay(response, writer);

        servlet.doGet(null, response);

        verify(response, writer);
    }

    @ObjectFactory
    public IObjectFactory getObjectFactory() {
        return new PowerMockObjectFactory();
    }
}
