/*
 *  Copyright (c) 2007 Mockito contributors
 *  This program is made available under the terms of the MIT License.
 */
package org.powermock.api.mockito.repackaged.cglib.transform;


import org.powermock.api.mockito.repackaged.cglib.core.ClassEmitter;

abstract public class ClassEmitterTransformer extends ClassEmitter implements ClassTransformer {
}
