/*
 *   Copyright 2016 the original author or authors.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */
package org.powermock.api.mockito.internal.expectation;

import org.powermock.api.mockito.expectation.PrivatelyExpectedArguments;

import java.lang.reflect.Method;

public class DefaultPrivatelyExpectedArguments implements PrivatelyExpectedArguments {

    private final Method method;
    private final Object mock;

    public DefaultPrivatelyExpectedArguments(Object mock, Method method) {
        this.mock = mock;
        this.method = method;
        method.setAccessible(true);
    }

    @Override
    public <T> void withArguments(Object firstArgument, Object... additionalArguments) throws Exception {
        if (additionalArguments == null || additionalArguments.length == 0) {
            method.invoke(mock, firstArgument);
        } else {
            Object[] allArgs = new Object[additionalArguments.length + 1];
            allArgs[0] = firstArgument;
            if (additionalArguments.length > 0) {
                System.arraycopy(additionalArguments, 0, allArgs, 1, additionalArguments.length);
            }
            method.invoke(mock, allArgs);
        }
    }

    @Override
    public <T> void withNoArguments() throws Exception {
        method.invoke(mock);
    }
}
