package samples.powermockito.junit4.bugs.github583;

/**
 *
 */
public class ParenClass {
    private int a;
    public int getA() {
        return a;
    }
    public void setA(int a) {
        this.a = a;
    }
}
