/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package samples.junit4.whitebox;

import org.junit.Test;
import org.powermock.reflect.internal.WhiteboxImpl;
import samples.whitebox.ClassWithPowerMockGeneratedConstructor;

import java.lang.reflect.Constructor;

import static org.junit.Assert.assertEquals;

public class PowerMockConstructorFiltrationTest {
	@Test
	public void findUniqueConstructorOrThrowExceptionFiltersPowerMockConstructors() throws Exception {
		Constructor<?> actualConstructor = WhiteboxImpl.findUniqueConstructorOrThrowException(ClassWithPowerMockGeneratedConstructor.class,
				(Object) null);
		assertEquals(ClassWithPowerMockGeneratedConstructor.class.getConstructor(String.class), actualConstructor);
	}
}
