/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package samples.overloading;

import samples.classhierarchy.ChildA;
import samples.classhierarchy.Parent;

public class OverloadedMethodsExample {
    public static void overloadedMethodWithOneArgument(Parent parent) {
    }

    public static void overloadedMethodWithOneArgument(ChildA child) {
    }

    public static void overloadedMethodWithTwoArguments(Parent parent, ChildA child) {
    }

    public static void overloadedMethodWithTwoArguments(Parent parent1, Parent parent2) {
    }
}
