/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package samples.junit3;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.powermock.modules.junit3.PowerMockSuite;
import samples.junit3.privateandfinal.PrivateFinalTest;
import samples.junit3.privateandfinal.StupidPrivateFinalTest;
import samples.junit3.singleton.MockStaticTest;
import samples.junit3.suppressconstructor.SuppressConstructorHierarchyDemoTest;

public class AllJUnit3Tests extends TestCase {

	@SuppressWarnings("unchecked")
	public static TestSuite suite() throws Exception {
		return new PowerMockSuite(StupidPrivateFinalTest.class, PrivateFinalTest.class, MockStaticTest.class,
				SuppressConstructorHierarchyDemoTest.class);
	}
}
