package org.powermock.hamcrest;

/**
 * A very simple class that's used just to assert that PowerMock works together
 * with Hamcrest.
 */
public class HamcrestExample {

	public static String getString() {
		return "hamcrest";
	}
}
