/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package samples.suppressconstructor;

import java.util.ArrayList;
import java.util.List;

public class AppaleList {
		
	public AppaleList() {
        throw new IllegalArgumentException("Shouldn't be called");
	}

	public AppaleList(String str) {
        throw new IllegalArgumentException("Shouldn't be called");
	}

	public String getAll() {
		List<String> list = new ArrayList<String>();
		list.add("sb1");
		List<String> list1 = new ArrayList<String>(10);
		list1.add("sb2");
		list1.add("sb3");
		list1.add("sb4");
		return "str";
	}
}
