/*
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.powermock.tests.utils.impl;

import org.powermock.core.MockRepository;
import org.powermock.core.classloader.annotations.SuppressStaticInitializationFor;
import org.powermock.tests.utils.TestClassesExtractor;

import java.lang.reflect.AnnotatedElement;
import java.util.LinkedList;
import java.util.List;

/**
 * Implementation of the {@link TestClassesExtractor} interface for classes that
 * should have their static initializers suppressed.
 * 
 */
public class StaticConstructorSuppressExtractorImpl extends AbstractTestClassExtractor {

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String[] getClassesToModify(AnnotatedElement element) {
		List<String> all = new LinkedList<String>();

		final SuppressStaticInitializationFor suppressAnnotation = element.getAnnotation(SuppressStaticInitializationFor.class);

		if (suppressAnnotation == null) {
			return null;
		} else {
			final String[] value = suppressAnnotation.value();
			for (String classToSuppress : value) {
				if (!"".equals(classToSuppress)) {
					all.add(classToSuppress);
					MockRepository.addSuppressStaticInitializer(classToSuppress);
				}
			}
		}

		return all.toArray(new String[0]);
	}
}
