/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package samples.junit4.resetmock;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;
import samples.singleton.StaticService;

import static org.powermock.api.easymock.PowerMock.*;

/**
 * Asserts that it works to reset mocks for static methods even after verify has
 * been invoked.
 */
@RunWith(PowerMockRunner.class)
@PrepareForTest(StaticService.class)
public class ResetForStaticMethodsTest {

	@Test
	public void assertThatResetWorksForStaticMethods() throws InterruptedException {
		mockStatic(StaticService.class);

		StaticService.sayHello();
		expectLastCall().once();

		replay(StaticService.class);

		StaticService.sayHello();

		verify(StaticService.class);

		reset(StaticService.class);

		StaticService.sayHello();
		expectLastCall().once();
	}
}
