package powermock.examples.partialmocking;

/**
 * Created by Katharina Laube on 08.09.2014.
 */
public class EntityManager {
    public void persist(Status status) {
        // persist status
    }
}
