/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package samples.staticinitializer;

import java.util.HashSet;
import java.util.Set;

public class StaticInitializerExample {

    private static final Set<String> mySet;

    static {
        mySet = new HashSet<String>();
        if (true) {
            throw new RuntimeException("This code must be suppressed!");
        }
    }

    public static Set<String> getMySet() {
        return mySet;
    }
}
