/*
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.powermock.core;

import java.util.AbstractSet;
import java.util.Iterator;
import java.util.Map;

public class IdentityHashSet<E> extends AbstractSet<E> {

    protected final Map<E, Boolean> backedMap;

    public IdentityHashSet() {
        this.backedMap = new ListMap<E, Boolean>();
    }

    @Override
    public int size() {
        return backedMap.size();
    }

    @Override
    public boolean contains(Object o) {
        return backedMap.containsKey(o);
    }

    @Override
    public boolean add(E o) {
        return backedMap.put(o, Boolean.TRUE) == null;
    }

    @Override
    public Iterator<E> iterator() {
        return backedMap.keySet().iterator();
    }

    @Override
    public boolean remove(Object o) {
        return backedMap.remove(o) != null;
    }

    @Override
    public void clear() {
        backedMap.clear();
    }
}
