/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.powermock.examples.dom4j;

public final class XMLProtocol {

	private XMLProtocol() {
	}

	public static final String ENCODE_ELEMENT = "EncodeElement";

	public static final String HEADER_ELEMENT = "MyHeader";

	public static final String HEADER_MSG_ID_ATTRIBUTE = "MyMsgIdAttribute";

	public static final String BODY_ELEMENT = "BodyElement";

}
