package samples.powermockito.junit4.bugs.github352;

import org.junit.Test;

/**
 *
 */
public abstract class MyAbstractTest {


    @Test
    public abstract void  test1();

    @Test
    public abstract void test2();

}
