/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package samples.expectnew;

import samples.Service;

public class ExpectNewWithMultipleCtorDemo {

    private final Service service;
    private final int times;

    public ExpectNewWithMultipleCtorDemo(Service service) {
        this(service, 1);
    }

    public ExpectNewWithMultipleCtorDemo(Service service, int times) {
        this.service = service;
        this.times = times;
    }

    public String useService() {
        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < times; i++) {
            builder.append(service.getServiceMessage());
        }
        return builder.toString();
    }
}
