/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package samples.constructor;

import samples.Service;

/**
 * Demonstrates the ability to invoke a test on a private constructor as well as
 * lazy initialization of private collaborators.
 * 
 * @author Johan Haleby
 */
public class PublicConstructorDemo {

	private Service privateService;

	private String constructorResult;

	public PublicConstructorDemo(String completeName) {
		constructorResult = privateService.getServiceMessage() + completeName;
	}

	public String getConstructorResult() {
		return constructorResult;
	}
}
